@@ -14,6 +14,7 @@
 import functools
 import ipaddress
 
+from cliff import columns
 from openstackclient.identity import common as identity_common
 from osc_lib import exceptions as osc_exc
 from osc_lib import utils
@@ -111,9 +112,11 @@ def _find_resource(list_funct, resource_name, root_tag, name, parent=None):
             raise osc_exc.CommandError(msg)
 
         # Try by ID next
-        resource = list_funct(*parent_args, id=name)[root_tag]
-        if len(resource) == 1:
-            return resource[0]
+        # Availability Zones don't support the id parameter
+        if resource_name != "availability_zones":
+            resource = list_funct(*parent_args, id=name)[root_tag]
+            if len(resource) == 1:
+                return resource[0]
 
     # We didn't find what we were looking for, raise a consistent error.
     msg = "Unable to locate {0} in {1}".format(name, resource_name)
@@ -213,6 +216,16 @@ def handle_additional_vips(vips, client_manager):
     return additional_vips
 
 
+def handle_vip_sg_ids(vip_sgs, client_manager):
+    vip_sg_ids = []
+    for sg in vip_sgs:
+        sg_id = get_resource_id(
+            client_manager.neutronclient.list_security_groups,
+            'security_groups', sg)
+        vip_sg_ids.append(sg_id)
+    return vip_sg_ids
+
+
 def get_loadbalancer_attrs(client_manager, parsed_args):
     attr_map = {
         'name': ('name', str),
@@ -269,6 +282,11 @@ def get_loadbalancer_attrs(client_manager, parsed_args):
             functools.partial(
                 handle_additional_vips, client_manager=client_manager)
         ),
+        'vip_sg_id': (
+            'vip_sg_ids',
+            functools.partial(
+                handle_vip_sg_ids, client_manager=client_manager),
+        )
     }
     add_tags_attr_map(attr_map)
 
@@ -685,16 +703,31 @@ def format_list(data):
     return '\n'.join(i['id'] for i in data)
 
 
+class ListColumn(columns.FormattableColumn):
+    def human_readable(self):
+        return format_list(self._value)
+
+
 def format_list_flat(data):
     return '\n'.join(i for i in data)
 
 
+class FlatListColumn(columns.FormattableColumn):
+    def human_readable(self):
+        return format_list_flat(self._value)
+
+
 def format_hash(data):
     if data:
         return '\n'.join('{}={}'.format(k, v) for k, v in data.items())
     return None
 
 
+class HashColumn(columns.FormattableColumn):
+    def human_readable(self):
+        return format_hash(self._value)
+
+
 def _format_kv(data):
     formatted_kv = {}
     values = data.split(',')
